/*-------------------------------------------------------------------------------------------------------------------*\
|  Copyright (C) 2015-2016 PayPal                                                                                     |
|                                                                                                                     |
|  Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance     |
|  with the License.                                                                                                  |
|                                                                                                                     |
|  You may obtain a copy of the License at                                                                            |
|                                                                                                                     |
|       http://www.apache.org/licenses/LICENSE-2.0                                                                    |
|                                                                                                                     |
|  Unless required by applicable law or agreed to in writing, software distributed under the License is distributed   |
|  on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for  |
|  the specific language governing permissions and limitations under the License.                                     |
\*-------------------------------------------------------------------------------------------------------------------*/

package com.paypal.selion.ios.sample;

import java.io.File;
import java.net.URL;
import java.util.EnumMap;

import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.paypal.selion.annotations.MobileTest;
import com.paypal.selion.configuration.Config;
import com.paypal.selion.platform.mobile.ios.GestureOptions;
import com.paypal.selion.platform.mobile.ios.UIAElement;
import com.paypal.selion.platform.mobile.ios.UIANavigationBar;

/*
 * DEVNOTE Tests in this class exist primarily for demonstration purposes and as a basic sanity checks.
 */
public class IOSDriverButtonTouchTest {

    private static final String appFolder = "/apps";

    @BeforeClass
    public void setup() {
        URL url = IOSDriverButtonTouchTest.class.getResource(appFolder);
        Config.setConfigProperty(Config.ConfigProperty.MOBILE_APP_FOLDER, (new File(url.getPath()).getAbsolutePath()));
    }

    @MobileTest(appName = "PageObjects", device = "iphone:8.1", deviceType = "iPhone5s")
    @Test
    public void testTwoFingerTap() throws InterruptedException {
        UIANavigationBar navigationBar = null;
        for (int i = 0; i < 2; i++) {
            navigationBar = new UIANavigationBar("xpath=//UIAApplication[1]/UIAWindow[1]/UIANavigationBar[1]");
            navigationBar.clickRightButton();
            Thread.sleep(500);
        }
        UIAElement twoFingerButton = new UIAElement("xpath=//UIAApplication[1]/UIAWindow[1]/UIAElement[1]");
        EnumMap<GestureOptions, String> options = new EnumMap<>(GestureOptions.class);
        options.put(GestureOptions.TAP_COUNT, "1");
        options.put(GestureOptions.TOUCH_COUNT, "2"); // transforms to Two Finger Tap
        options.put(GestureOptions.DURATION, "0");
        twoFingerButton.tapWithOptions(options);
        // twoFingerButton.doubleTap(); // also works
        twoFingerButton.tapWithOptions(options);
        Thread.sleep(2 * 1000);
    }

    @AfterClass
    public void teardown() {
        Config.setConfigProperty(Config.ConfigProperty.MOBILE_APP_FOLDER,
                Config.ConfigProperty.MOBILE_APP_FOLDER.getDefaultValue());
    }

}